<?php declare(strict_types=1);

namespace App\Entity;

use App\Repository\FoodOrderRepository;
use DateTimeImmutable;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\IdGenerator\UlidGenerator;
use Symfony\Bridge\Doctrine\Types\UlidType;
use Symfony\Component\Uid\Ulid;

#[ORM\Entity(repositoryClass: FoodOrderRepository::class)]
class FoodOrder
{
    #[ORM\Id]
    #[ORM\GeneratedValue(strategy: 'CUSTOM')]
    #[ORM\Column(type: UlidType::NAME, unique: true)]
    #[ORM\CustomIdGenerator(class: UlidGenerator::class)]
    private Ulid|null $id = null;

    #[ORM\Column(nullable: true)]
    private DateTimeImmutable|null $closedAt = null;

    #[ORM\ManyToOne(inversedBy: 'foodOrders')]
    #[ORM\JoinColumn(nullable: false)]
    private FoodVendor|null $foodVendor = null;

    /**
     * @var Collection<int, OrderItem>
     */
    #[ORM\OneToMany(targetEntity: OrderItem::class, mappedBy: 'foodOrder', orphanRemoval: true)]
    private Collection $orderItems;

    public function __construct()
    {
        $this->orderItems = new ArrayCollection;
    }

    public function getId(): Ulid|null
    {
        return $this->id;
    }

    public function getCreatedAt(): DateTimeImmutable
    {
        return $this->id->getDateTime();
    }

    public function getClosedAt(): DateTimeImmutable|null
    {
        return $this->closedAt;
    }

    public function setClosedAt(DateTimeImmutable|null $closedAt): static
    {
        $this->closedAt = $closedAt;

        return $this;
    }

    public function isClosed(): bool
    {
        return $this->closedAt instanceof DateTimeImmutable;
    }

    public function close(): static
    {
        return $this->setClosedAt(new DateTimeImmutable);
    }

    public function open(): static
    {
        return $this->setClosedAt(null);
    }

    public function getFoodVendor(): FoodVendor|null
    {
        return $this->foodVendor;
    }

    public function setFoodVendor(FoodVendor|null $foodVendor): static
    {
        $this->foodVendor = $foodVendor;

        return $this;
    }

    /**
     * @return Collection<int, OrderItem>
     */
    public function getOrderItems(): Collection
    {
        return $this->orderItems;
    }

    public function addOrderItem(OrderItem $orderItem): static
    {
        if (! $this->orderItems->contains($orderItem)) {
            $this->orderItems->add($orderItem);
            $orderItem->setFoodOrder($this);
        }

        return $this;
    }

    public function removeOrderItem(OrderItem $orderItem): static
    {
        // set the owning side to null (unless already changed)
        if ($this->orderItems->removeElement($orderItem) && $orderItem->getFoodOrder() === $this) {
            $orderItem->setFoodOrder(null);
        }

        return $this;
    }
}
